package ink.wgink.module.examine.service.paper.question.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.examine.dao.paper.question.IPaperQuestionDao;
import ink.wgink.module.examine.pojo.dtos.paper.question.PaperQuestionDTO;
import ink.wgink.module.examine.pojo.dtos.question.QuestionDTO;
import ink.wgink.module.examine.service.paper.question.IPaperQuestionService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.util.UUIDUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Description: 试卷试题
 * @Author: WenG
 * @Date: 2022/5/23 23:44
 * @Version: 1.0
 **/
@Service
public class PaperQuestionServiceImpl extends DefaultBaseService implements IPaperQuestionService {

    @Autowired
    private IPaperQuestionDao paperQuestionDao;

    @Override
    public void save(String paperId, List<String> questionIds) {
        Map<String, Object> params = getHashMap(8);
        params.put("paperId", paperId);
        params.put("questionSort", "ZZZ-000");
        for (String questionId : questionIds) {
            params.put("paperQuestionId", UUIDUtil.getUUID());
            params.put("questionId", questionId);
            paperQuestionDao.save(params);
        }
    }

    @Override
    public List<PaperQuestionDTO> list(String paperId, Map<String, Object> params) {
        params = params == null ? getHashMap(0) : params;
        return paperQuestionDao.list(params);
    }

    @Override
    public SuccessResultList<List<PaperQuestionDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<PaperQuestionDTO> paperQuestionDTOS = paperQuestionDao.list(page.getParams());
        PageInfo<PaperQuestionDTO> pageInfo = new PageInfo<>(paperQuestionDTOS);
        return new SuccessResultList<>(paperQuestionDTOS, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public SuccessResultList<List<PaperQuestionDTO>> listPage(String paperId, ListPage page) {
        page.getParams().put("paperId", paperId);
        return listPage(page);
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageQuestionByExcludePaperId(String excludePaperId, ListPage page) {
        page.getParams().put("excludePaperId", excludePaperId);
        PageHelper.startPage(page.getPage(), page.getRows());
        List<QuestionDTO> questionDTOS = paperQuestionDao.listQuestion(page.getParams());
        PageInfo<QuestionDTO> pageInfo = new PageInfo<>(questionDTOS);
        return new SuccessResultList<>(questionDTOS, pageInfo.getPageNum(), pageInfo.getTotal());
    }
}
